"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLogger = exports.LogLevel = void 0;
const util_1 = require("util");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["TRACE"] = 100] = "TRACE";
    LogLevel[LogLevel["DEBUG"] = 200] = "DEBUG";
    LogLevel[LogLevel["INFO"] = 300] = "INFO";
    LogLevel[LogLevel["WARNING"] = 400] = "WARNING";
    LogLevel[LogLevel["ERROR"] = 500] = "ERROR";
    LogLevel[LogLevel["CRITICAL"] = 600] = "CRITICAL";
    LogLevel[LogLevel["AUDIT"] = 999] = "AUDIT";
    LogLevel[LogLevel["AUDIT_ALERT"] = 999] = "AUDIT_ALERT";
    LogLevel[LogLevel["SUPPRESS"] = 1000] = "SUPPRESS";
})(LogLevel || (exports.LogLevel = LogLevel = {}));
/**
 * Returns a new logger instance given the options provided.
 * @param {LoggerOptions} options
 */
function getLogger(options) {
    const defaultSettings = {
        level: _resolveLogLevel(options?.level, process.env.LOG_LEVEL),
        prefix: process.env.LOG_PREFIX ?? '',
        timestamp: process.env.LOG_TIMESTAMP === 'true' || false,
        callerInfo: process.env.LOG_CALLERINFO === 'true' || false,
    };
    const settings = Object.assign(defaultSettings, options || {});
    const masks = new Map();
    let hasMasks = false;
    return {
        trace: (message, attachment) => {
            _log(LogLevel.TRACE, message, attachment);
        },
        debug: (message, attachment) => {
            _log(LogLevel.DEBUG, message, attachment);
        },
        info: (message, attachment) => {
            _log(LogLevel.INFO, message, attachment);
        },
        warning: (message, attachment) => {
            _log(LogLevel.WARNING, message, attachment);
        },
        error: (message, attachment) => {
            _log(LogLevel.ERROR, message, attachment);
        },
        critical: (message, attachment) => {
            _log(LogLevel.CRITICAL, message, attachment);
        },
        audit: (message, attachment) => {
            _log(LogLevel.AUDIT, message, attachment);
        },
        auditAlert: (message, attachment) => {
            _log(LogLevel.AUDIT_ALERT, message, attachment);
        },
        getSettings: () => {
            return settings;
        },
        isDebugEnabled: () => {
            return settings.level <= LogLevel.DEBUG;
        },
        isTraceEnabled: () => {
            return settings.level <= LogLevel.TRACE;
        },
        addMask(value, placeholder = '***') {
            masks.set(value, placeholder);
            hasMasks = true;
        },
        removeMask(value) {
            masks.delete(value);
            hasMasks = masks.size > 0;
        },
        clearMasks() {
            masks.clear();
            hasMasks = false;
        },
    };
    function _log(logLevel, message, attachment) {
        if (settings.level <= logLevel) {
            let formattedMessage;
            if (settings.timestamp) {
                formattedMessage = (0, util_1.format)('[%s] %s %s%s', LogLevel[logLevel], new Date().toISOString(), settings.prefix, message);
            }
            else {
                formattedMessage = (0, util_1.format)('[%s] %s%s', LogLevel[logLevel], settings.prefix, message);
            }
            if (settings.callerInfo) {
                attachment = _attachCallerInfo(attachment);
            }
            if (hasMasks) {
                formattedMessage = _mask(formattedMessage);
            }
            /* tslint:disable: no-console */
            if (attachment) {
                let stringAttachment = (0, util_1.inspect)(attachment, false, null);
                if (hasMasks) {
                    stringAttachment = _mask(stringAttachment);
                }
                console.log(formattedMessage, stringAttachment);
            }
            else {
                console.log(formattedMessage);
            }
            /* tslint:enable: no-console */
        }
        function _mask(input) {
            for (const [maskValue, maskPlaceholder] of masks.entries()) {
                input = input.replaceAll(maskValue, maskPlaceholder);
            }
            return input;
        }
    }
}
exports.getLogger = getLogger;
/**
 * Resolves the log level for the logger instance.
 * Log level is resolved in the following order:
 * 1. Provided logLevel assuming that it is set and valid
 * 2. From process.env.LOG_LEVEL, assuming that it is set and valid
 * 3. Default LogLevel.INFO
 * @param level Optional specific log level
 * @param processLogLevel
 */
function _resolveLogLevel(level, processLogLevel) {
    if (level && LogLevel[level]) {
        return level;
    }
    else if (processLogLevel !== undefined && LogLevel[processLogLevel]) {
        return LogLevel[processLogLevel];
    }
    return LogLevel.INFO;
}
/* istanbul ignore next */
function _attachCallerInfo(attachment) {
    if (attachment == null) {
        attachment = {};
    }
    const callerStackLine = (new Error()).stack?.split('\n')[4];
    if (callerStackLine) {
        const pattern = /\((.*):(\d+):(\d+)\)$/; // This regex pattern matches the file path, line number, and column number
        const match = pattern.exec(callerStackLine);
        if (match) {
            attachment.callerInfo = {
                file: match[1],
                line: match[2],
            };
        }
    }
    return attachment;
}
